==> wordpress_0.6.97-2.tar.gz: wordpress-0.6.97/src/wpmenugraphicswidget.cpp <==
/*
 * wpgraphicsmenu.cpp
 *
 *  Created on: 18.5.2010
 *      Author: jlep
 */

#include <QGraphicsSceneMouseEvent>
#include <QFontMetrics>
#include <QPropertyAnimation>

#include "wpmenugraphicswidget.h"
#include "PopupMenu.h"

WPMenuGraphicsWidget::WPMenuGraphicsWidget(BlogView& aBlogView, QGraphicsWidget* aParent,
    QGraphicsScene& aScene)
        : QGraphicsWidget(aParent),
        iBlogView(aBlogView),
        iScene(aScene),
        iPopupMenu(NULL),
        iAnimation(NULL),
        iMode(EOptionsQuit),
        iHappyAction(NULL),
        iSadAction(NULL),
        iFullMenuPixmap(NULL),
        iMousePressed(0)
{
    setFocusPolicy(Qt::StrongFocus);
    setMinimumHeight(MenuHeight);
}

WPMenuGraphicsWidget::~WPMenuGraphicsWidget()
{
    delete iPopupMenu;
    delete iFullMenuPixmap;
}

void WPMenuGraphicsWidget::addAction(QAction* aAction)
{
    if (!iPopupMenu) {
        iPopupMenu = new PopupMenu;
        connect(iPopupMenu, SIGNAL(aboutToHide()), this, SLOT(MenuHiding()));
    }
    iPopupMenu->addAction(aAction);
    iActions.append(aAction);
    iPopupMenu->setWindowOpacity(0.6);
}

void WPMenuGraphicsWidget::setButtonActions(QAction* aHappyAction, QAction* aSadAction)
{
    iHappyAction = aHappyAction;
    iSadAction = aSadAction;
}

void WPMenuGraphicsWidget::setMode(TMode aMode)
{
    if (iMode == aMode) return;

    iMode = aMode;
    QGraphicsSceneResizeEvent event;
    event.setNewSize(QSizeF(boundingRect().width(),boundingRect().height()));
    resizeEvent(&event);
    update();
}

void WPMenuGraphicsWidget::showMenu()
{
    // move to below statusbar.
    iPopupMenu->show();
    iPopupMenu->setWindowOpacity(0.6);
    iPopupMenu->setFocus();
    if (!iAnimation)
    {
        iAnimation = new QPropertyAnimation(iPopupMenu, "geometry", this);
        iAnimation->setDuration(200);

        QPoint menuPoint = QPoint(0, StatusBarHeight);
        
        int startx = menuPoint.x() - iPopupMenu->width();
        int starty = menuPoint.y();
        int endx = menuPoint.x();
        int endy = menuPoint.y();
    
        qDebug("WP::Animating iPopupMenu FROM %d, %d TO %d, %d", startx, starty, endx, endy);
    
        iAnimation->setStartValue(QRect(startx,starty,iPopupMenu->width(),iPopupMenu->height()));
        iAnimation->setEndValue(QRect(endx,endy,iPopupMenu->width(),iPopupMenu->height()));
        iAnimation->setEasingCurve(QEasingCurve::Linear);
        connect(iAnimation, SIGNAL(finished()), this, SLOT(AnimationFinished()));
        iAnimation->start();
    }
}

void WPMenuGraphicsWidget::resizeEvent(QGraphicsSceneResizeEvent *event)
{
    delete iFullMenuPixmap;
    iFullMenuPixmap = NULL;
    // Device in Portrait, height is fixed
    // Menubar in portrait has higher width than height, hence the '>'
    //
    if (event->newSize().width() > event->newSize().height())
    {
        iFullMenuPixmap = new QPixmap(event->newSize().width(),MenuHeight);
        QPainter painter(iFullMenuPixmap);
        // EOptionsQuit
        //
        if (iMode == EOptionsQuit)
        {

            QPixmap& px = *iBlogView.GetThemeData().iOptionsPixmap;
            int width = event->newSize().width();
            width -= px.width();
            width /= 2;

            // paint the pixmap ready.
            //
            painter.drawPixmap(0,0,width,px.height(),*iBlogView.GetThemeData().iMenuPixmap);
            painter.drawPixmap(width,0,*iBlogView.GetThemeData().iOptionsPixmap);
            painter.drawPixmap(width+px.width(),0,width,px.height(),*iBlogView.GetThemeData().iMenuPixmap);
        }
        // ESaveCancel
        //
        else
        {
            // draw full menu bar first
            //
            QPixmap& menuFillingPixmap = *iBlogView.GetThemeData().iMenuPixmap;
            painter.drawPixmap(0,0,event->newSize().width(),menuFillingPixmap.height(),menuFillingPixmap);

            int x(0);
            if (iMode != EBack)
            {
                x = event->newSize().width()/4;
                x -= iBlogView.GetThemeData().iSavePixmap->width()/2;
                painter.drawPixmap(x,0,*iBlogView.GetThemeData().iSavePixmap);
            }
            
            x = 3*(event->newSize().width()/4);
            x -= iBlogView.GetThemeData().iSavePixmap->width()/2;
            painter.drawPixmap(x,0,*iBlogView.GetThemeData().iCancelPixmap);
        }
    }
    // Device in Landscape, width is fixed
    //
    else
    {
        iFullMenuPixmap = new QPixmap(MenuHeight,event->newSize().height());
        QPainter painter(iFullMenuPixmap);
        // move origin to the bottom left.
        //
        painter.translate(QPoint(0,event->newSize().height()));
        // rotate 90 degrees left.
        //
        painter.rotate(-90);

        QPixmap& menuFillingPixmap = *iBlogView.GetThemeData().iMenuPixmap;

        // EOptionsQuit
        //
        if (iMode == EOptionsQuit)
        {
            int width = event->newSize().height(); // height is width for the area, cause rotated.
            QPixmap& px = *iBlogView.GetThemeData().iOptionsPixmap;
            width -= px.width();
            width /= 2;
            painter.drawPixmap(0,0,width,px.height(),menuFillingPixmap);
            painter.drawPixmap(width,0,*iBlogView.GetThemeData().iOptionsPixmap);
            painter.drawPixmap(width+px.width(),0,width,px.height(),menuFillingPixmap);
        }
        // ESaveCancel
        //
        else
        {
            // draw full menu bar first
            //
            painter.drawPixmap(0,0,event->newSize().height(),menuFillingPixmap.height(),menuFillingPixmap);

            int x(0);
            if (iMode != EBack)
            {
                x = event->newSize().height()/4;
                x -= iBlogView.GetThemeData().iSavePixmap->width()/2;
                painter.drawPixmap(x,0,*iBlogView.GetThemeData().iSavePixmap);
            }

            x = 3*(event->newSize().height()/4);
            x -= iBlogView.GetThemeData().iSavePixmap->width()/2;
            painter.drawPixmap(x,0,*iBlogView.GetThemeData().iCancelPixmap);
        }
    }
}

void WPMenuGraphicsWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem*/*option*/,
    QWidget*/*widget*/)
{
    painter->drawPixmap(0,0,*iFullMenuPixmap);
    if (iMousePressed > 0)
    {
        painter->setBrush(QColor(255,255,255,128));
        if (iMousePressed == 3)
        {
            painter->drawRect(0,0,boundingRect().width(),boundingRect().height());
        }
        else if (boundingRect().width() > boundingRect().height())
        {
            if (iMousePressed == 1)
            {
                painter->drawRect(0,0,boundingRect().width()/2,boundingRect().height());
            }
            else
            {
                painter->drawRect(boundingRect().width()/2,0,boundingRect().width()/2,boundingRect().height());
            }
        }
        else
        {
            if (iMousePressed == 1)
            {
                painter->drawRect(0,boundingRect().height()/2,boundingRect().width(),boundingRect().height()/2);
            }
            else
            {
                painter->drawRect(0,0,boundingRect().width(),boundingRect().height()/2);
            }
        }
    }
}

void WPMenuGraphicsWidget::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    iMousePressed = CalculateButton(event);
    update();
}

void WPMenuGraphicsWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
{
    if (iMousePressed == 0) return;
    else if (iMousePressed == 1 && iMousePressed == CalculateButton(event)) iHappyAction->trigger();
    else if (iMousePressed == 2 && iMousePressed == CalculateButton(event)) iSadAction->trigger();
    // no else
    
    if (iMousePressed != 3)
    {
        iMousePressed = 0;
        return;
    }
    
    iMousePressed = 0;
    showMenu();
    event->accept();
    update();
}

int WPMenuGraphicsWidget::CalculateButton(QGraphicsSceneMouseEvent* event)
{
    // Simple button heuristics to avoid use of two widgets here.
    //
    if (iMode != EOptionsQuit)
    {
        if (boundingRect().width() > boundingRect().height())
        {
            if (event->pos().x() < boundingRect().width()/2)
            {
                if (iHappyAction) return 1;
            }
            else
            {
                if (iSadAction) return 2;
            }
        }
        else
        {
            if (event->pos().y() > boundingRect().height()/2)
            {
                if (iHappyAction) return 1;
            }
            else
            {
                if (iSadAction) return 2;
            }
        }
        return 0;
    }
    else
    {
        return 3;
    }
}

void WPMenuGraphicsWidget::MenuHiding()
{
    iPopupMenu->hide();
}

void WPMenuGraphicsWidget::AnimationFinished()
{
    delete iAnimation;
    iAnimation = NULL;
}

==> wordpress_0.6.99.tar.gz: wordpress-0.6.99/src/wpmenugraphicswidget.cpp <==
/*
 * wpgraphicsmenu.cpp
 *
 *  Created on: 18.5.2010
 *      Author: jlep
 */

#include <QGraphicsSceneMouseEvent>
#include <QFontMetrics>
#include <QPropertyAnimation>

#include "wpmenugraphicswidget.h"
#include "PopupMenu.h"

WPMenuGraphicsWidget::WPMenuGraphicsWidget(BlogView& aBlogView, QGraphicsWidget* aParent,
    QGraphicsScene& aScene)
        : QGraphicsWidget(aParent),
        iBlogView(aBlogView),
        iScene(aScene),
        iPopupMenu(NULL),
        iAnimation(NULL),
        iMode(EOptionsQuit),
        iHappyAction(NULL),
        iSadAction(NULL),
        iFullMenuPixmap(NULL),
        iMousePressed(0)
{
    setFocusPolicy(Qt::StrongFocus);
    setMinimumHeight(MenuHeight);
}

WPMenuGraphicsWidget::~WPMenuGraphicsWidget()
{
    delete iPopupMenu;
    delete iFullMenuPixmap;
}

void WPMenuGraphicsWidget::addAction(QAction* aAction)
{
    if (!iPopupMenu) {
        iPopupMenu = new PopupMenu;
        connect(iPopupMenu, SIGNAL(aboutToHide()), this, SLOT(MenuHiding()));
    }
    iPopupMenu->addAction(aAction);
    iActions.append(aAction);
    iPopupMenu->setWindowOpacity(0.6);
}

void WPMenuGraphicsWidget::setButtonActions(QAction* aHappyAction, QAction* aSadAction)
{
    iHappyAction = aHappyAction;
    iSadAction = aSadAction;
}

void WPMenuGraphicsWidget::setMode(TMode aMode)
{
    if (iMode == aMode) return;

    iMode = aMode;
    QGraphicsSceneResizeEvent event;
    event.setNewSize(QSizeF(boundingRect().width(),boundingRect().height()));
    resizeEvent(&event);
    update();
}

void WPMenuGraphicsWidget::showMenu()
{
    // move to below statusbar.
    iPopupMenu->show();
    iPopupMenu->setWindowOpacity(0.6);
    iPopupMenu->setFocus();
    if (!iAnimation)
    {
        iAnimation = new QPropertyAnimation(iPopupMenu, "geometry", this);
        iAnimation->setDuration(200);

        QPoint menuPoint = QPoint(0, StatusBarHeight);
        
        int startx = menuPoint.x() - iPopupMenu->width();
        int starty = menuPoint.y();
        int endx = menuPoint.x();
        int endy = menuPoint.y();
    
        qDebug("WP::Animating iPopupMenu FROM %d, %d TO %d, %d", startx, starty, endx, endy);
    
        iAnimation->setStartValue(QRect(startx,starty,iPopupMenu->width(),iPopupMenu->height()));
        iAnimation->setEndValue(QRect(endx,endy,iPopupMenu->width(),iPopupMenu->height()));
        iAnimation->setEasingCurve(QEasingCurve::Linear);
        connect(iAnimation, SIGNAL(finished()), this, SLOT(AnimationFinished()));
        iAnimation->start();
    }
}

void WPMenuGraphicsWidget::resizeEvent(QGraphicsSceneResizeEvent *event)
{
    delete iFullMenuPixmap;
    iFullMenuPixmap = NULL;
    // Device in Portrait, height is fixed
    // Menubar in portrait has higher width than height, hence the '>'
    //
    if (event->newSize().width() > event->newSize().height())
    {
        iFullMenuPixmap = new QPixmap(int(event->newSize().width()),MenuHeight);
        QPainter painter(iFullMenuPixmap);
        // EOptionsQuit
        //
        if (iMode == EOptionsQuit)
        {

            QPixmap& px = *iBlogView.GetThemeData().iOptionsPixmap;
            int width = int(event->newSize().width());
            width -= px.width();
            width /= 2;

            // paint the pixmap ready.
            //
            painter.drawPixmap(0,0,width,px.height(),*iBlogView.GetThemeData().iMenuPixmap);
            painter.drawPixmap(width,0,*iBlogView.GetThemeData().iOptionsPixmap);
            painter.drawPixmap(width+px.width(),0,width,px.height(),*iBlogView.GetThemeData().iMenuPixmap);
        }
        // ESaveCancel
        //
        else
        {
            // draw full menu bar first
            //
            QPixmap& menuFillingPixmap = *iBlogView.GetThemeData().iMenuPixmap;
            painter.drawPixmap(0,0,int(event->newSize().width()),menuFillingPixmap.height(),menuFillingPixmap);

            int x(0);
            if (iMode != EBack)
            {
                x = int(event->newSize().width()/4);
                x -= iBlogView.GetThemeData().iSavePixmap->width()/2;
                painter.drawPixmap(x,0,*iBlogView.GetThemeData().iSavePixmap);
            }
            
            x = int(3*(event->newSize().width()/4));
            x -= iBlogView.GetThemeData().iSavePixmap->width()/2;
            painter.drawPixmap(x,0,*iBlogView.GetThemeData().iCancelPixmap);
        }
    }
    // Device in Landscape, width is fixed
    //
    else
    {
        iFullMenuPixmap = new QPixmap(MenuHeight,int(event->newSize().height()));
        QPainter painter(iFullMenuPixmap);
        // move origin to the bottom left.
        //
        painter.translate(QPoint(0,int(event->newSize().height())));
        // rotate 90 degrees left.
        //
        painter.rotate(-90);

        QPixmap& menuFillingPixmap = *iBlogView.GetThemeData().iMenuPixmap;

        // EOptionsQuit
        //
        if (iMode == EOptionsQuit)
        {
            int width = int(event->newSize().height()); // height is width for the area, cause rotated.
            QPixmap& px = *iBlogView.GetThemeData().iOptionsPixmap;
            width -= px.width();
            width /= 2;
            painter.drawPixmap(0,0,width,px.height(),menuFillingPixmap);
            painter.drawPixmap(width,0,*iBlogView.GetThemeData().iOptionsPixmap);
            painter.drawPixmap(width+px.width(),0,width,px.height(),menuFillingPixmap);
        }
        // ESaveCancel
        //
        else
        {
            // draw full menu bar first
            //
            painter.drawPixmap(0,0,int(event->newSize().height()),menuFillingPixmap.height(),menuFillingPixmap);

            int x(0);
            if (iMode != EBack)
            {
                x = int(event->newSize().height()/4);
                x -= iBlogView.GetThemeData().iSavePixmap->width()/2;
                painter.drawPixmap(x,0,*iBlogView.GetThemeData().iSavePixmap);
            }

            x = int(3*(event->newSize().height()/4));
            x -= iBlogView.GetThemeData().iSavePixmap->width()/2;
            painter.drawPixmap(x,0,*iBlogView.GetThemeData().iCancelPixmap);
        }
    }
}

void WPMenuGraphicsWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem*/*option*/,
    QWidget*/*widget*/)
{
    painter->drawPixmap(0,0,*iFullMenuPixmap);
    if (iMousePressed > 0)
    {
        painter->setBrush(QColor(255,255,255,128));
        if (iMousePressed == 3)
        {
            painter->drawRect(0,0,int(boundingRect().width()),int(boundingRect().height()));
        }
        else if (boundingRect().width() > boundingRect().height())
        {
            if (iMousePressed == 1)
            {
                painter->drawRect(0,0,int(boundingRect().width()/2),int(boundingRect().height()));
            }
            else
            {
                painter->drawRect(int(boundingRect().width()/2),0,int(boundingRect().width()/2),int(boundingRect().height()));
            }
        }
        else
        {
            if (iMousePressed == 1)
            {
                painter->drawRect(0,int(boundingRect().height()/2),int(boundingRect().width()),int(boundingRect().height()/2));
            }
            else
            {
                painter->drawRect(0,0,int(boundingRect().width()),int(boundingRect().height()/2));
            }
        }
    }
}

void WPMenuGraphicsWidget::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    iMousePressed = CalculateButton(event);
    update();
}

void WPMenuGraphicsWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
{
    if (iMousePressed == 0) return;
    else if (iMousePressed == 1 && iMousePressed == CalculateButton(event)) iHappyAction->trigger();
    else if (iMousePressed == 2 && iMousePressed == CalculateButton(event)) iSadAction->trigger();
    // no else
    
    if (iMousePressed != 3)
    {
        iMousePressed = 0;
        return;
    }
    
    iMousePressed = 0;
    showMenu();
    event->accept();
    update();
}

int WPMenuGraphicsWidget::CalculateButton(QGraphicsSceneMouseEvent* event)
{
    // Simple button heuristics to avoid use of two widgets here.
    //
    if (iMode != EOptionsQuit)
    {
        if (boundingRect().width() > boundingRect().height())
        {
            if (event->pos().x() < boundingRect().width()/2)
            {
                if (iHappyAction) return 1;
            }
            else
            {
                if (iSadAction) return 2;
            }
        }
        else
        {
            if (event->pos().y() > boundingRect().height()/2)
            {
                if (iHappyAction) return 1;
            }
            else
            {
                if (iSadAction) return 2;
            }
        }
        return 0;
    }
    else
    {
        return 3;
    }
}

void WPMenuGraphicsWidget::MenuHiding()
{
    iPopupMenu->hide();
}

void WPMenuGraphicsWidget::AnimationFinished()
{
    delete iAnimation;
    iAnimation = NULL;
}
